@@ -1,5 +1,5 @@
 /* NBD client library in userspace
- * Copyright (C) 2013-2019 Red Hat Inc.
+ * Copyright (C) 2013-2020 Red Hat Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,6 +31,7 @@
 #endif
 
 #include "internal.h"
+#include "vector.h"
 
 int
 nbd_unlocked_set_tls (struct nbd_handle *h, int tls)
@@ -107,11 +108,13 @@ nbd_unlocked_set_tls_username (struct nbd_handle *h, const char *username)
   return 0;
 }
 
+DEFINE_VECTOR_TYPE (string, char)
+
 char *
 nbd_unlocked_get_tls_username (struct nbd_handle *h)
 {
   char *s, *ret;
-  size_t len;
+  string str = empty_vector;
 
   if (h->tls_username) {
     ret = strdup (h->tls_username);
@@ -137,31 +140,22 @@ nbd_unlocked_get_tls_username (struct nbd_handle *h)
     return ret;
   }
 
-  len = 16;
-  ret = NULL;
-  while (ret == NULL) {
-    char *oldret = ret;
-
-    ret = realloc (oldret, len);
-    if (ret == NULL) {
-      set_error (errno, "realloc");
-      free (oldret);
+  for (;;) {
+    if (getlogin_r (str.ptr, str.alloc) == 0) {
+      return str.ptr;
+    }
+    else if (errno != ERANGE) {
+      set_error (errno, "getlogin_r");
+      free (str.ptr);
       return NULL;
     }
-
-    if (getlogin_r (ret, len) != 0) {
-      if (errno == ERANGE) {
-        /* Try again with a larger buffer. */
-        len *= 2;
-        continue;
-      }
-      set_error (errno, "getlogin_r");
-      free (ret);
+    /* Try again with a larger buffer. */
+    if (string_reserve (&str, str.alloc == 0 ? 16 : str.alloc * 2) == -1) {
+      set_error (errno, "realloc");
+      free (str.ptr);
       return NULL;
     }
   }
-
-  return ret;
 }
 
 int
@@ -230,9 +224,21 @@ tls_get_fd (struct socket *sock)
   return sock->u.tls.oldsock->ops->get_fd (sock->u.tls.oldsock);
 }
 
-/* XXX Calling gnutls_bye is possible, but it may send and receive
- * data over the wire which would require modifications to the state
- * machine.  So instead we abruptly drop the TLS session.
+static bool
+tls_shut_writes (struct nbd_handle *h, struct socket *sock)
+{
+  int r = gnutls_bye (sock->u.tls.session, GNUTLS_SHUT_WR);
+
+  if (r == GNUTLS_E_AGAIN || r == GNUTLS_E_INTERRUPTED)
+    return false;
+  if (r != 0)
+    debug (h, "ignoring gnutls_bye failure: %s", gnutls_strerror (r));
+  return sock->u.tls.oldsock->ops->shut_writes (h, sock->u.tls.oldsock);
+}
+
+/* XXX Calling gnutls_bye(GNUTLS_SHUT_RDWR) is possible, but it may send
+ * and receive data over the wire which would require further modifications
+ * to the state machine.  So instead we abruptly drop the TLS session.
  */
 static int
 tls_close (struct socket *sock)
@@ -254,6 +260,7 @@ static struct socket_ops crypto_ops = {
   .send = tls_send,
   .pending = tls_pending,
   .get_fd = tls_get_fd,
+  .shut_writes = tls_shut_writes,
   .close = tls_close,
 };
 
