@@ -1,5 +1,5 @@
 /* NBD client library in userspace
- * Copyright (C) 2013-2019 Red Hat Inc.
+ * Copyright (C) 2013-2020 Red Hat Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -65,16 +65,17 @@ nbd_create (void)
   h->tls_verify_peer = true;
   h->request_sr = true;
 
-  h->uri_allow_transports = (uint32_t) -1;
+  h->uri_allow_transports = LIBNBD_ALLOW_TRANSPORT_MASK;
   h->uri_allow_tls = LIBNBD_TLS_ALLOW;
   h->uri_allow_local_file = false;
 
-  h->gflags = (LIBNBD_HANDSHAKE_FLAG_FIXED_NEWSTYLE |
-               LIBNBD_HANDSHAKE_FLAG_NO_ZEROES);
+  h->gflags = LIBNBD_HANDSHAKE_FLAG_MASK;
 
   s = getenv ("LIBNBD_DEBUG");
   h->debug = s && strcmp (s, "1") == 0;
 
+  h->strict = LIBNBD_STRICT_MASK;
+
   h->public_state = STATE_START;
   h->state = STATE_START;
   h->pid = -1;
@@ -112,8 +113,6 @@ nbd_create (void)
 void
 nbd_close (struct nbd_handle *h)
 {
-  struct meta_context *m, *m_next;
-
   nbd_internal_set_error_context ("nbd_close");
 
   if (h == NULL)
@@ -125,15 +124,13 @@ nbd_close (struct nbd_handle *h)
   nbd_unlocked_clear_debug_callback (h);
 
   free (h->bs_entries);
-  for (m = h->meta_contexts; m != NULL; m = m_next) {
-    m_next = m->next;
-    free (m->name);
-    free (m);
-  }
+  nbd_internal_reset_size_and_flags (h);
+  nbd_internal_free_option (h);
   free_cmd_list (h->cmds_to_issue);
   free_cmd_list (h->cmds_in_flight);
   free_cmd_list (h->cmds_done);
-  nbd_internal_free_string_list (h->argv);
+  string_vector_iter (&h->argv, (void *) free);
+  free (h->argv.ptr);
   if (h->sa_sockpath) {
     if (h->pid > 0)
       kill (h->pid, SIGTERM);
@@ -157,7 +154,8 @@ nbd_close (struct nbd_handle *h)
   free (h->tls_certificates);
   free (h->tls_username);
   free (h->tls_psk_file);
-  nbd_internal_free_string_list (h->request_meta_contexts);
+  string_vector_iter (&h->request_meta_contexts, (void *) free);
+  free (h->request_meta_contexts.ptr);
   free (h->hname);
   pthread_mutex_destroy (&h->lock);
   free (h);
@@ -226,12 +224,69 @@ nbd_unlocked_get_export_name (struct nbd_handle *h)
   return copy;
 }
 
+int
+nbd_unlocked_set_full_info (struct nbd_handle *h, bool request)
+{
+  h->full_info = request;
+  return 0;
+}
+
+int
+nbd_unlocked_get_full_info (struct nbd_handle *h)
+{
+  return h->full_info;
+}
+
+char *
+nbd_unlocked_get_canonical_export_name (struct nbd_handle *h)
+{
+  char *r;
+
+  if (h->eflags == 0) {
+    set_error (EINVAL, "server has not returned export flags, "
+               "you need to connect to the server first");
+    return NULL;
+  }
+  if (h->canonical_name == NULL) {
+    set_error (ENOTSUP, "server did not advertise a canonical name");
+    return NULL;
+  }
+
+  r = strdup (h->canonical_name);
+  if (r == NULL) {
+    set_error (errno, "strdup");
+    return NULL;
+  }
+  return r;
+}
+
+char *
+nbd_unlocked_get_export_description (struct nbd_handle *h)
+{
+  char *r;
+
+  if (h->eflags == 0) {
+    set_error (EINVAL, "server has not returned export flags, "
+               "you need to connect to the server first");
+    return NULL;
+  }
+  if (h->description == NULL) {
+    set_error (ENOTSUP, "server did not advertise a description");
+    return NULL;
+  }
+
+  r = strdup (h->description);
+  if (r == NULL) {
+    set_error (errno, "strdup");
+    return NULL;
+  }
+  return r;
+}
+
 int
 nbd_unlocked_add_meta_context (struct nbd_handle *h, const char *name)
 {
   char *copy;
-  size_t len;
-  char **list;
 
   if (strnlen (name, NBD_MAX_STRING + 1) > NBD_MAX_STRING) {
     set_error (ENAMETOOLONG, "meta context name too long for NBD protocol");
@@ -243,22 +298,47 @@ nbd_unlocked_add_meta_context (struct nbd_handle *h, const char *name)
     set_error (errno, "strdup");
     return -1;
   }
-  len = h->request_meta_contexts == NULL ? 0
-    : nbd_internal_string_list_length (h->request_meta_contexts);
-  list = realloc (h->request_meta_contexts,
-                  sizeof (char *) * (len+2 /* + new entry + NULL */));
-  if (list == NULL) {
+
+  if (string_vector_append (&h->request_meta_contexts, copy) == -1) {
     free (copy);
     set_error (errno, "realloc");
     return -1;
   }
-  h->request_meta_contexts = list;
-  list[len] = copy;
-  list[len+1] = NULL;
 
   return 0;
 }
 
+ssize_t
+nbd_unlocked_get_nr_meta_contexts (struct nbd_handle *h)
+{
+  return h->request_meta_contexts.size;
+}
+
+char *
+nbd_unlocked_get_meta_context (struct nbd_handle *h, size_t i)
+{
+  char *ret;
+
+  if (i >= h->request_meta_contexts.size) {
+    set_error (EINVAL, "meta context request out of range");
+    return NULL;
+  }
+
+  ret = strdup (h->request_meta_contexts.ptr[i]);
+  if (ret == NULL)
+    set_error (errno, "strdup");
+
+  return ret;
+}
+
+int
+nbd_unlocked_clear_meta_contexts (struct nbd_handle *h)
+{
+  string_vector_iter (&h->request_meta_contexts, (void *) free);
+  string_vector_reset (&h->request_meta_contexts);
+  return 0;
+}
+
 int
 nbd_unlocked_set_request_structured_replies (struct nbd_handle *h,
                                              bool request)
@@ -296,6 +376,20 @@ nbd_unlocked_get_handshake_flags (struct nbd_handle *h)
   return h->gflags;
 }
 
+int
+nbd_unlocked_set_strict_mode (struct nbd_handle *h, uint32_t flags)
+{
+  h->strict = flags;
+  return 0;
+}
+
+/* NB: may_set_error = false. */
+uint32_t
+nbd_unlocked_get_strict_mode (struct nbd_handle *h)
+{
+  return h->strict;
+}
+
 const char *
 nbd_unlocked_get_package_name (struct nbd_handle *h)
 {
